#include <Reg51.h>
#define D1 255

sbit tDQ    = P2^0;
sbit tCLK   = P2^1;
sbit tRST   = P2^2;

unsigned char data tmpbuffer;
unsigned char data tmpsign;

void delay1(void);
void initUART(void);

void init3wires(void);
void StartConvert1620(void);
void StopConvert1620(void);

void send1620cmd(void);
void read1620cmd(void);
void ReadTemp(void);
void send1620Temperature(void);

//void UARTisr(void) interrupt 4 using 1;

void main(void) {
  //(ENABLE SERIAL INTERRUPT)
  IE    = 0X90; // 10010000 BIT.7=EA  BIT.5=ES
  initUART();
  while (1); // wait for UART interrupt
}

void UARTisr(void) interrupt 4 using 1 {

  if (RI==1)
  {
    RI=0;
    switch (SBUF) {

      case '1' : // 1 = read current temperature
        init3wires();
	ReadTemp();
        // send temperature to PC
        TI=0;	// CLEAR LAST TX INTERRUPT 
        SBUF=tmpbuffer;
        while (~TI);
        StopConvert1620();
        break;

      case '2' : // 2 = read TL 

        delay1();
        tmpbuffer = 0xA2;  //  read TL  command
        send1620cmd();
        read1620cmd();
        delay1();
         TI=0;	// CLEAR LAST TX INTERRUPT 
         SBUF=tmpbuffer;
         while (~TI);
         
        break;

      case '3' : // 3 = read TH

        delay1();
        tmpbuffer = 0xA1;  // read TH  command
        send1620cmd();
        read1620cmd();
        delay1();
         TI=0;	// CLEAR LAST TX INTERRUPT 
         SBUF=tmpbuffer;
         while (~TI);

        break;


      case '4' : // 4 = set TL = + 10 degree

        delay1();
        tmpbuffer = 0x02;  // send write TL  command
        send1620cmd();
        tmpbuffer = 20;  // TL = + 10 degree
        tmpsign = 0;
        send1620Temperature();
        delay1();
        break;

      case '5' : // 5 = set TL = + 25 degree

        delay1();
        tmpbuffer = 0x02;  // send write TL  command
        send1620cmd();
        tmpbuffer = 50;  // TL = + 25 degree
        tmpsign = 0;
        send1620Temperature();
        delay1();
        break;


      case '6' : // 6 = set TH = + 28 degree

        delay1();
        tmpbuffer = 0x01;  // send write TH  command
        send1620cmd();
        tmpbuffer = 56;  // TH = + 28 degree
        tmpsign = 0;
        send1620Temperature();
        delay1();
        break;

      case '7' : // 7 = set TH = + 30 degree

        delay1();
        tmpbuffer = 0x01;  // send write TH  command
        send1620cmd();
        tmpbuffer = 60;  // TH = + 30 degree
        tmpsign = 0;
        send1620Temperature();
        delay1();
        break;

    }

  } //end if(RI==1)

  if (TI==1)
  {
    TI=0;
  }

}


void ReadTemp() {
    delay1();
    tmpbuffer = 0xAA;  // send read temperature command
    send1620cmd();
    read1620cmd();
}    

void StopConvert1620() {
    delay1();
    tmpbuffer = 0x22;  // send stop convert temperature command
    send1620cmd();
    tRST = 0;	     // set DS1620 inactive
    delay1();
}    

void StartConvert1620() {
  delay1();
  tmpbuffer = 0xEE;  // send start convert temperature
  send1620cmd();
  tRST = 0;	     // set DS1620 inactive
  delay1();
}


void init3wires() {
  // initialize DS1620 as 3 wires , continues mode.  
  P2 = 0x00;
  tCLK = 0;
  tmpbuffer = 0x0C;  // send write configuration
  send1620cmd();
  tmpbuffer = 0x02;  // set CPU bit = 1, 3 wires mode
  send1620cmd();
  tRST = 0;	     // set DS1620 inactive
  StartConvert1620();
  // initialize DS1620 as 3 wires , continues mode.  
}



void initUART() {
  // SET MODE2 FOR TIMER1
  TMOD	= 0x20; // SET MODE2 FOR TIMER1  00100000B
  TH1	= 0xFD; // 38400bps TIMER1 22.1MHZ RESTART VALUE   
  TL1	= 0xFD; // AUTO RELOAD TL1 IF TL1=00H
  // UART TRANSMIT AND RECEIVE ENABLE
  SCON	= 0x52;	// SCON MODE 1 FOR UART, REN=1	01010010B
  PCON	= 0x00; // SET PCON.SMOD = 0     00000000B
  // START UART
  TR1	= 1;	// TCON.TR1 = 1 , TIMER1 START NOW  
}


void send1620cmd() {
  unsigned char data i,b=0x01;
  tRST = 1;                // set DS1620 enable
  delay1();
  for(i=0;i<8;i++) {
    tCLK = 0;                 // SETUP FOR CLOCK PLUSE
    tDQ = ( tmpbuffer & b );  // MOVE a BIT TO DQ 
    tCLK = 1;                 // FINISH THE CLOCK PULSE 
    b = (b<<1);
  }
}

void send1620Temperature() {
  unsigned char data i,b=0x01;
  tRST = 1;                // set DS1620 enable
  delay1();
  for(i=0;i<8;i++) {
    tCLK = 0;                 // SETUP FOR CLOCK PLUSE
    tDQ = ( tmpbuffer & b );  // MOVE a BIT TO DQ 
    tCLK = 1;                 // FINISH THE CLOCK PULSE 
    b = (b<<1);
  }
    tCLK = 0;                 // SETUP FOR CLOCK PLUSE
    tDQ = tmpsign;	      // MOVE a BIT TO DQ 
    tCLK = 1;                 // FINISH THE CLOCK PULSE 
}


void read1620cmd() {
  unsigned char data i,b=1;
  tmpbuffer = 0;
  delay1();
  for(i=0;i<8;i++) {
    tCLK=0;        // set CLK low
    if (tDQ) tmpbuffer|=b;   // READ A Bit
    tCLK=1;        // set CLK high
    b = (b<<1);
  }
  tCLK=0;          // set CLK low
  tmpsign = tDQ;   // READ sign
  tCLK=1;          // set CLK high  
  tRST=0;          // set RST DS1620 = disable
  delay1();
}



void delay1() {
  unsigned char data  i,k;
  for (i=0;i<D1;i++) {
      for (k=0;k<D1;k++); 
  }
}
