;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Author : VR2AAA	Date : 26 Aug 2002
;
; Program Description
;
; 1. Setup 3 wires mode in DS1620
; 2. Setup temperature conversion mode in DS1620
; 3. setup 8051 timer1 and uart
; 4.    read temperature from DS1620
; 5.    Send temperature via 8051 uart
; 6. goto 4.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Visual Basic (MSComm) <--> PC RS232 (rx,tx) <--> 8051 UART (P3.0/1)
;       8051 (P2.0-DQ, P2.1-CLK, P2.2-RST) <-->  DS1620
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
	DQ    BIT   P2.5
	CLK   BIT   P2.6
	RST   BIT   P2.7


	ORG	0000H
	;SETUP DS1620  P2.0=DQ   P2.1=CLK   P2.2=RST
	MOV	P2,#00H		;CLEAR PORT 2
	MOV	A,#00H		;CLEAR REGISTER A.
	;
	SETB	CLK		;ACTIVE P2.1 - DS1620 CLK
	;
	MOV	A,#00CH		;SEND WRITE CONFIGURATION COMMAND
	ACALL	CMDOUT		
	MOV	A,#00000010B	;SEND WRITE CPU BIT=1 , 3WIRE MODE ENABLE
	ACALL	CMDOUT
	CLR	RST		;DEACTIVATE THE DS1620
	ACALL	DELAY		;DELAY FOR EEPROM PROG. CYCLE
;
	MOV	A,#0EEH		;SEND WRITE START CONVERSION COMMAND
	ACALL	CMDOUT
	CLR	RST		;SET P2.2 LOW - INACTIVE DS1620 RST
	ACALL	DELAY		;DELAY FOR EEPROM PROG. CYCLE	
;	
	;SETUP TIMER 89C51
	MOV	TMOD,#00100000B	;SET MODE2 FOR TIMER1
	MOV     PCON,#00000000B	;SET PCON.SMOD = 0 
	MOV	TH1,#0FDH	;38400bps TIMER1 22.1MHZ RESTART VALUE
	MOV	TL1,#0FDH	;AUTO RELOAD TL1 IF TL1=00H
	SETB	TR1		;TCON.TR1 = 1 , TIMER1 START NOW
	;SETUP 89C51 SERIAL OUTPUT
	MOV	SCON,#01000010B	;SCON MODE 1 FOR UART, REN=0	
LOOP:	ACALL	DELAY		;WAIT FOR 0.1ms
	MOV	A,#0AAH		;SEND READ TEMPATURE COMMAND 
	ACALL	CMDOUT		;
	ACALL	RDTEMP		;START READING
	CLR	TI		;CLEAR LAST TX INTERRUPT 
	MOV	SBUF,A		;SEND VALUE TO SERIAL BUFFER FROM A
WAIT:	JNB	TI,WAIT		;WAIT UNTIL TI BIT IS SET - TRANSFER FINISHED
        JMP	LOOP
	;;;;;;;;;;;;;;;;;;;;;;;
	;SUBROUTINES 
	;;;;;;;;;;;;;;;;;;;;;;;
	;DELAY SUBROUTINE 67.9 ms  fOSC 22.1MHZ
DELAY:	MOV	R4,#20 
DL0:	MOV	R5,#250
DL1:	MOV	R6,#200
DL2:	DJNZ	R6,DL2
	DJNZ	R5,DL1
	DJNZ	R4,DL0
	RET
	;DELAY1 SUBROUTINE 13.6 ms  fOSC 22.1MHZ
DELAY1:	MOV	R5,#250
DL11:	MOV	R6,#200
DL21:	DJNZ	R6,DL21
	DJNZ	R5,DL11
	RET	
	;2.7ms   fOSC 22.1MHZ
DELAY2:	MOV	R5,#100
DL12:	MOV	R6,#100
DL22:	DJNZ	R6,DL22
	DJNZ	R5,DL12
	RET
	;TRANSMIT 8BITS COMMAND TO DS1620
CMDOUT:	MOV	R1,#8		;Send 8 bits CONFIG COMMAND	
	SETB	RST		;TAKE RST HIGH , START DS1620 COMMUNICATION
	ACALL	DELAY1
LOOP1:	RRC	A		;SHIFT MOST RIGHT BIT INTO CARRY BIT C - LSB FIRST.
	CLR	CLK		;SETUP FOR CLOCK PLUSE
	MOV	DQ,C		;MOVE CARRY BIT C TO DQ	
	SETB	CLK		;FINISH THE CLOCK PULSE 
	DJNZ	R1,LOOP1	;LOOP 8 BITS 
	RET
	;WRITE TEMPERATURE TH TL  ANSWER = A 00000000 + C 0
TMPOUT:	MOV	R1,#8		;SEND 9 BITS TEMPERATURE TO DS1620
	SETB	RST		;TAKE RST HIGH , START DS1620 COMMUNICATION
	ACALL	DELAY1
LOOP2:	RRC	A		;SHIFT MOST RIGHT BIT INTO CARRY BIT C - LSB FIRST.
	CLR	CLK		;SETUP FOR CLOCK PLUSE
	MOV	DQ,C		;MOVE CARRY BIT C TO DQ 
	SETB	CLK		;FINISH THE PULSE  
	DJNZ	R1,LOOP2	;LOOP 8 BITS 
	CLR	CLK		;SETUP FOR CLOCK PLUSE
	CLR	DQ		;MOVE 0 TO DQ, THE 9TH BIT, + ALWAYS
	SETB	CLK		;FINISH THE CLOCK PULSE 
	ACALL	DELAY2
	RET
	;READ TEMPERATURE  ANSWER = A 00000000 + C 0
RDTEMP:	MOV	R1,#9
	ACALL	DELAY2
	CLR	C		;CLEAR CARRY BIT
LOOP3:	RRC	A		;ROTATE A + C RIGHT
	CLR	CLK		;SETUP FOR CLOCK PLUSE
	MOV	C,DQ		;READ A BIT
	SETB	CLK		;FINISH THE PULSE
	DJNZ	R1,LOOP3	;LOOP 9 BIT
	CLR	RST		;STOP RST DS1620
	ACALL	DELAY1
	RET
	;
	END
